!******************************************************************************
! FILE: mpi_mm.f
! DESCRIPTION:
!   MPI Matrix Multiply - Fortran Version
!   In this code, the master task distributes a matrix multiply
!   operation to numtasks-1 worker tasks.
!   NOTE1:  C and Fortran versions of this code differ because of the way
!   arrays are stored/passed.  C arrays are row-major order but Fortran
!   arrays are column-major order.
! AUTHOR: Blaise Barney. Adapted from Ros Leibensperger, Cornell Theory
!   Center. Converted to MPI: George L. Gusciora, MHPCC (1/95)
! LAST REVISED: 04/02/05
!******************************************************************************

      program mm
      include 'mpif.h'

      parameter (NRA = 500)
      parameter (NCA = 500)
      parameter (NCB = 500)
      parameter (MASTER = 0)
      parameter (FROM_MASTER = 1)
      parameter (FROM_WORKER = 2)

      integer 	numtasks,taskid,numworkers,source,dest,mtype, &
               cols,avecol,extra, offset,i,j,k,ierr
      integer status(MPI_STATUS_SIZE)
      real*8	a(NRA,NCA), b(NCA,NCB), c(NRA,NCB)

      call MPI_INIT( ierr )
      call MPI_COMM_RANK( MPI_COMM_WORLD, taskid, ierr )
      call MPI_COMM_SIZE( MPI_COMM_WORLD, numtasks, ierr )
      numworkers = numtasks-1
      print *, 'task ID= ',taskid
      
! *************************** master task *************************************
      if (taskid .eq. MASTER) then

!     Initialize A and B 
        do  i=1, NRA
          do  j=1, NCA
             a(i,j) = (i-1)+(j-1)
          enddo 
        enddo

        do i=1, NCA
          do  j=1, NCB
            b(i,j) = (i-1)*(j-1)
          enddo
        enddo

!     Send matrix data to the worker tasks
!
!  cols = no. of cols to send to each worker
!  offset = next position in B matrix
!  
 
        avecol = NCB/numworkers
        extra = mod(NCB, numworkers)
        offset = 1
        mtype = FROM_MASTER

        do  dest=1, numworkers
          if (dest .le. extra) then
            cols = avecol + 1
          else
            cols = avecol
          endif
          write(*,*)'   sending',cols,' cols to task',dest

! Here we want 4 MPI sends

          ! call mpi_send(offset,...)
          ! call mpi_send(cols,...)
          ! call mpi_send(a,..)
          ! call mpi_send(b(1, offset), ..)

          offset = offset + cols
       enddo ! 1 .. numworkers 

!     Receive results from worker tasks
        mtype = FROM_WORKER
        do  i=1, numworkers
          source = i

!   Here 3 receives

          ! call mpi_recv(offset,)
          ! call mpi_recv(cols,..)
          ! call mpi_recv(c(1,offset),..)

        enddo 

!     Print results
      write(*,*) 'C(MAX,MAX)=',c(ncb,ncb)

      endif


! *************************** worker task *************************************
      if (taskid >  MASTER) then
!     Receive matrix data from master task
        mtype = FROM_MASTER

!  Four receives from master

        ! call mpi_recv(offset,..)
        ! call mpi_recv(cols,..)
        ! call mpi_recv(a,..)
        ! call mpi_recv(b,cols*NCA,..)

!     Do matrix multiply
        do k=1, cols
          do  i=1, NRA
            c(i,k) = 0.0
            do  j=1, NCA
              c(i,k) = c(i,k) + a(i,j) * b(j,k)
            enddo
          enddo
        enddo 

!     Send results back to master task
        mtype = FROM_WORKER

        ! call mpi_send(offset,)
        ! call mpi_send(cols,)
        ! call mpi_send(c,..)

      endif

      call MPI_FINALIZE(ierr)

      end program 
